/**********************************************************************************/
/*                                                                                */
/* File: mandelbrot.cpp                                                           */
/* Author: bkenwright@xbdev.net                                                   */
/* URL: www.xbdev.net                                                             */
/* Date: 11-08-2005                                                               */
/* Desciption: Mandelbrot Fractal Texture Class                                   */
/*                                                                                */
/* Updated: 11-08-2005 by bkenwright                                              */
/*                                                                                */
/**********************************************************************************/


#include "mandelbrot.h"
#include "math.h"


////////////////////////////////////////////////////////////////////////////////////
//                                                                                //
////////////////////////////////////////////////////////////////////////////////////
HRESULT CMandelbrot::Create(IDirect3DDevice8 ** pDevice)
{
	// Basic error checking
	if( *pDevice == NULL )
	{
		MessageBox(0, "Error in CMandelbrot Create", "Error - NULL Pointer", 0 );
		return S_FALSE;
	}//End if(..)

	m_pd3dDevice = *pDevice;

	// BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG *//
	// Well not really a bug - but something to be added to make the class more   //
	// flexible, as we create a 512x512 texture - maybe alter the creation        //
	// parameters so we can create different size textures later.                 //
	// BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG ** BUG *//
	m_pd3dDevice->CreateTexture(512, 512, 1
                                 , D3DUSAGE_RENDERTARGET, D3DFMT_X8R8G8B8
                                 , D3DPOOL_DEFAULT, &m_pTexture );

	D3DXFillTexture(m_pTexture, MakTexture, NULL);

	return S_OK;
}//End Create(..)


////////////////////////////////////////////////////////////////////////////////////
//                                                                                //
////////////////////////////////////////////////////////////////////////////////////
HRESULT CMandelbrot::Release()
{
	if( m_pTexture != NULL ) m_pTexture->Release();
	return S_OK;
}//End Release()


////////////////////////////////////////////////////////////////////////////////////
//                                                                                //
////////////////////////////////////////////////////////////////////////////////////
void CMandelbrot::MakTexture(D3DXVECTOR4* pOut, D3DXVECTOR2 *pTexCoord, D3DXVECTOR2 *pTexelSize, LPVOID pData)
{

	// int pTexCoord->x
	// int pTexCoord->y

	// pOut->x   -> red
	// pOut->y   -> green
	// pOut->z   -> blue
	// pOut->w   -> alpha

	int x = static_cast<int>(512 * pTexCoord->x);
    int y = static_cast<int>(512 * pTexCoord->y);
	int width  = 512;
	int height = 512;
    
	int it;

	static int maxIter=100;

    double tr, ti;

    // Pixel scaling: map a 4x4 area with (0,0) at the center
    double cr=  ((double)x- (width/2))    /width*4;
    double ci= -((double)y- (height/2))   /height*4;

	// x==0
	// Then width/2 / width/4 
	// example values.   -512/2  / 512/4  = -256 / 128 = -2
	// x==256
	// example values. 256-256  / 512/4  = -128 / 128 = 0
	// x==512
	// example values. 512-256  / 512/4  = 256 / 128 = 2

	// NOTE:
	// Basically i is for Y, and r for X axis

    // For the Mandelbrot set, we start fixing C=
    double r=0;
    double i=0;
    for(it=0; it<maxIter; ++it)
	{
        // Z=Z*Z+C
        tr=r*r-i*i+cr;
        ti=2*r*i+ci;
        r=tr;
        i=ti;

        // If the distance of the point is greater than 2, exit
        if( (r*r+i*i) > 4 )  break;
    }

	// Okay our iteration value is between 0 and maxIter.
	// We artificially boost colors
	float value = static_cast<float>(maxIter*sqrt((double)it/maxIter));

	// Note, here in this demo where using colours rgb between 0 and 1.0f.
	// But if you need to you can scale the value from 0 to 256
    float color_r = value; 
    float color_g = value; 
    float color_b = value;

	// PLOTTING THE PIXEL!
	// Its an awful lot of code as we have our red greed blue values...but
	// so it looks cooler...I've done it so that at different stages from
	// the center of the mandelbrot fractal shape as it moves outwards the
	// colours change differently giving a more shocking effect :)

	if( it > maxIter/2  )   // Very center a fixed colour
	{                       // This is the actual mandelbrot shape!!!  If you just do all 
    // Plot the pixel point colour
	pOut->x = 0.8f;         // the other parts below a fixed colour, this is the pixel
	pOut->y = 0.8f;         // plotting part that you want !...important this.
	pOut->z = 0.9f;
	pOut->w = 1.0f;
	}
	else if (it > maxIter/20) // Gradually moving out from the center
	{
	pOut->x = color_r; 
	pOut->y = 0.0f;
	pOut->z = 0.0f;
	pOut->w = 1.0f;
	}
	else if (it > maxIter/30) // More rounded edges
	{
	pOut->x = 0.0f; 
	pOut->y = color_g;
	pOut->z = 0.0f;
	pOut->w = 1.0f;
	}
	else if (it > maxIter/40) // Very edge..not even part of the mandelbrot shape practically
	{
	pOut->x = 0.0f; 
	pOut->y = 0.0f;
	pOut->z = color_b;
	pOut->w = 1.0f;
	}
	else
	{
	pOut->x = color_r; 
	pOut->y = color_g;
	pOut->z = color_b;
	pOut->w = 1.0f;
	}

}//End MakTexture(..)




